from hashlib import md5
from configparser import ConfigParser
from urllib.parse import urlencode
from typing import Optional

import json

from .http import HTTPRequest


class DHL:
    BASE_URL = "https://api-eu.dhl.com/track/"
    BASE_URL_SANDBOX = "https://api-test.dhl.com/track/"

    def __init__(self, key: str, secret: str, base_url: str = BASE_URL):
        self.key = key
        self.secret = secret
        self.base_url = base_url

    @classmethod
    def from_config(cls, config: ConfigParser | str, section: str = "DHL") -> "DHL":
        if isinstance(config, str):
            temp_config = ConfigParser()
            temp_config.read(config)
            config = temp_config

        key = config.get(section, "key")
        secret = config.get(section, "secret")
        base_url = config.get(section, "base_url", fallback=cls.BASE_URL)

        return cls(key, secret, base_url)

    def get_request(self, endpoint: str, add_token: bool = True) -> HTTPRequest:
        url = self.base_url + endpoint

        request = HTTPRequest(url)

        if add_token:
            request.add_header("DHL-API-Key", self.key)

        return request

    def track(self, tracking_number: str, **kwargs) -> bytes:
        params = {
            "trackingNumber": tracking_number,
        }

        for key in ("service", "requesterCountryCode", "originCountryCode", "language", "offset", "limit"):
            if key in kwargs:
                params[key] = kwargs[key]

        request = self.get_request(f"shipments?{urlencode(params)}")
        return request.execute()



from urllib.request import Request, urlopen
from urllib.error import HTTPError
from io import BytesIO

import gzip
import json


class HTTPRequest(Request):
    USER_AGENT = "Mozilla/5.0 (compatible; DHLTrack/dev; +https://kumig.it/kumitterer/dhltrack)"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.add_header("User-Agent", self.USER_AGENT)

    def execute(self, load_json: bool = True, *args, **kwargs):
        try:
            response = urlopen(self, *args, **kwargs).read()
        except HTTPError as e:
            print(e.read())
            raise
        if load_json:
            response = json.loads(response)
        return response



from unittest import TestCase, main
from configparser import ConfigParser

import json

from dhltrack import *

class TestHTTPRequest(TestCase):
    def test_http_request(self):
        http = HTTPRequest("https://httpbin.org/get")
        response = http.execute()
        self.assertEqual(response["headers"]["User-Agent"], http.USER_AGENT)

class TestDHL(TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.config = ConfigParser()
        self.config.read("config.ini")
        self.dhl = DHL.from_config(self.config)

    def test_tracking(self):
        tracking_number = "LE650235858DE"
        response = self.dhl.track(tracking_number)
        self.assertEqual(response["shipments"][0]["id"], tracking_number)
        
if __name__ == "__main__":
    main()